#!/bin/sh

#Add the tools in this directory to the path
PATH=$PATH:/Insight/bin:/Insight/sbin

if [ $# -ne 7 ];
then
	echo "You must provide all of the required data."
	echo "Usage:"
	echo "loadUpgrade_USB.sh <step> <ID key file> <machine type> <serial number> <output directory> <USB directory>"
	echo "Where:"
	echo "step: The step in the USB upgrade process (info or upgrade)."
	echo "	The info step populates the USB directory with the authentication data for this device. Auth data is recorded to the output directory."
	echo "  The upgrade step extracts data from the USB directory, decrypts it and writes it to the output directory."
		
	exit 1
fi

TYPE=$2
ID_KEY=$3
MACH_TYPE=$4
SERIAL_NUMBER=$5
OUTPUT_DIR=$6
USB_DIR=$7

DH_PARAMS=/Insight/keys/dhp.pem
DH_PRIV_KEY=/Insight/keys/dhkey.pem
DH_PUB_KEY=/Insight/keys/dhpubkey.pem


REQUEST_TAR=insightupdate.fwreq


#Find the newest insightupdate*.fw file on the usb drive so the user doesn't need to rename it when the download creates a insightupdate(2).fw or something similar
RESPONSE_TAR=$USB_DIR/insightupdate.fw
for f in $USB_DIR/*.fw; do
	[ -e "$f" ] || continue
    [ "$f" -nt "$RESPONSE_TAR" ] && RESPONSE_TAR="$f"
done
echo "Upgrading from $RESPONSE_TAR"

echo "Writing USB credentials to drive."

#Generate diffie-helman (local) keys (Only if they haven't been generated already).
if [ ! -e $DH_PRIV_KEY ];
then
	openssl genpkey -paramfile $DH_PARAMS -out $DH_PRIV_KEY #private key
	if [ $? -ne 0 ];
	then
		exit 1
	fi 

	openssl pkey -in $DH_PRIV_KEY -pubout -out $DH_PUB_KEY #public key
	if [ $? -ne 0 ];
	then
		exit 1
	fi 
fi
pubkey=$(cat $DH_PUB_KEY)
pubkey=${pubkey//$'\n'/} #Remove all newlines from the key

KEY_TEXT=$(openssl pkey -pubin -in $DH_PUB_KEY -text)
KEY_TEXT=${KEY_TEXT//$'\n'/}
PUB_KEY=$(echo $KEY_TEXT | sed 's/ //g' | grep -oE 'public-key:([0-9a-f]+:)+([0-9a-f])*' | sed 's/://g' | sed 's/public-key//')
PRIME=$(echo $KEY_TEXT | sed 's/ //g' | grep -oE 'prime:([0-9a-f]+:)+([0-9a-f])*' | sed 's/://g' | sed 's/prime//')

#Create a salt to obscure the ID key.
salt=$(openssl rand -base64 32)

#Get the identity key
key=$(cat $ID_KEY)
key=${key//$'\n'/} #Remove all newlines from the key

#Generate a digest to sign the outgoing packet.
digestTmp=$(mktemp)
printf $SERIAL_NUMBER$salt > $digestTmp
hmac=$(openssl dgst -sha1 -hex -hmac $key < $digestTmp | sed 's/^.* //')	

#Construct the outgoing package (for processing on the server)
packageTmp=$(mktemp)
echo "public:"$PUB_KEY > $packageTmp
echo "prime:"$PRIME >> $packageTmp
echo "serial:"$SERIAL_NUMBER >> $packageTmp
echo "machtype:"$MACH_TYPE >> $packageTmp
echo "salt:"$salt >> $packageTmp
echo "hmac:"$hmac >> $packageTmp
packet=$(./2json < $packageTmp)

OUTPUT_REQUEST=$SERIAL_NUMBER
echo $packet > $OUTPUT_REQUEST

#Output the json packet to a tarball accumulation of files.
if [ -e $USB_DIR/$REQUEST_TAR ];
then
	#Append the fw_request file to the fwrequest tarball.
#		tar -rvf $USB_DIR/$REQUEST_TAR $OUTPUT_REQUEST #Cant do this with busybox tar, but this is the behavior we want!

	#make a temporary ram drive to unpack & repack the request tarball (appending the file).
	tempdir=fwrequest_tmp
	mkdir $tempdir
	mount -t tmpfs -o size=8M tmpfs $tempdir
	lastdir=$(pwd)
	cd $tempdir
	tar -xvf $USB_DIR/$REQUEST_TAR
	cp $lastdir/$OUTPUT_REQUEST ./$OUTPUT_REQUEST
	tar -cvf $USB_DIR/$REQUEST_TAR ./*
	cd $lastdir
	umount $tempdir
	rm -rf $tempdir
	rm -rf $OUTPUT_REQUEST
else
	tar -cvf $USB_DIR/$REQUEST_TAR $OUTPUT_REQUEST
	rm $OUTPUT_REQUEST
fi
sync #Sync the USB drive after write to ensure everything made it to the disk.


#If there is a fw update tar on the drive, try upgrading from it.
if [ -e "$RESPONSE_TAR" ];
then

	echo "Upgrading from USB."
	
	#Validate that the upgrade package exists.
	if [ ! -e "$RESPONSE_TAR" ];
	then
		echo "No firmware file '$RESPONSE_TAR' was found on the USB drive!"
		exit 1
	fi
	
	#Unpack just the files for this device.
	configFile=$SERIAL_NUMBER.json
	peerkeyFile=$SERIAL_NUMBER.pem
	ivFile=$SERIAL_NUMBER.iv
	cipher=$SERIAL_NUMBER.pass
	
	tar -xvf "$RESPONSE_TAR" $configFile
	if [ $? -ne 0 ];
	then
		echo "Could not find the firmware file for this device ($SERIAL_NUMBER) in the firmware package!"
		exit 1
	fi
	
	upgradeVerTmp=$(mktemp)
	jextract --data $configFile --member version > $upgradeVerTmp
	ugpradeVersion=$(cat $upgradeVerTmp)
	version -t $ugpradeVersion

	if [ $? -eq 1 ];
	then
		echo "Upgrade code version $ugpradeVersion is up-level, continuing upgrade..."		
	else
		echo "Upgrade code version $ugpradeVersion is down-level with local code. Cancelling upgrade."
		exit 1
	fi
	
	#Extract the DH peer-key
	tar -xvf "$RESPONSE_TAR" $peerkeyFile
	tar -xvf "$RESPONSE_TAR" $ivFile
	
	#Generate the shared secret
	SECRET=$(openssl pkeyutl -derive -inkey $DH_PRIV_KEY -peerkey $peerkeyFile | hexdump -ve '1/1 "%.2x"')
	SECRET=${SECRET:0:64}
	
	#Decrypt the passphrase
	tar -xvf "$RESPONSE_TAR" $cipher
	PASSPHRASE=passphrase.usb
	openssl enc -aes-256-cbc -d -K $SECRET -iv $(cat $ivFile) -in $cipher -out $PASSPHRASE
	if [ $? -ne 0 ];
	then
		echo "Decrytion failed."
		exit 1
	fi
	
	#Decrypt the package file.
	packageFile=$(./jextract --data $configFile --member package)
	currentDir=$(pwd)
	cd $OUTPUT_DIR
	tar -xOf "$RESPONSE_TAR" $packageFile | openssl bf -d -pass file:$currentDir/$PASSPHRASE | tar -xzv
	cd $currentDir
	
	#Delete temporary files.
	rm $PASSPHRASE
	rm $configFile
	rm $peerkeyFile
	rm $ivFile
	rm $cipher
	
	#Delete the existing credentials to regeneration them the next time a USB stick is inserted.
	rm $DH_PRIV_KEY
	rm $DH_PUB_KEY
fi

exit
